#include "project-defs.h"
#include "DS18B20.h"

/*
 * Codes des commandes supportées par le DS18B20.
 */
typedef enum {
	DS18B20_convertT = 0x44,
	DS18B20_writeScratchpad = 0x4e,
	DS18B20_readScratchpad = 0xbe,
	DS18B20_copyScratchpad = 0x48,
	DS18B20_recallE2 = 0xb8,
	DS18B20_readPowerSupply = 0xb4,
} DS18B20Command;

// Prend 7.2ms si tout va bien.
bool ds18b20Initialise(DS18B20Sensor *sensor, DS18B20Precision precision) {
	bool result = onewireReset(sensor->oneWireBus);
	
	if (result) {
		// Un DS18B20 est présent sur le bus 1-wire => on lit
		// son identifiant unique (un seul DB18B20 => readROM
		// au lieu de searchROM).
		onewireReadROM(sensor->oneWireBus, sensor->_romCode);
		
		// Et on configure la précision de la conversion.
		onewireReset(sensor->oneWireBus);
		onewireMatchROM(sensor->oneWireBus, sensor->_romCode);
		sensor->_buffer[0] = 0;
		sensor->_buffer[1] = 0;
		sensor->_buffer[2] = 0x1f | (precision << 5);
		onewireSendCommand(sensor->oneWireBus, DS18B20_writeScratchpad);
		onewireSendBytes(sensor->oneWireBus, sensor->_buffer, 3);
	}
	
	return result;
}

// Prend 21.12ms + le temps d'exécution de onewireWaitForCompletion(),
// c'est-à-dire 93.75ms pour une résolution de 9 bits et jusqu'à 750ms
// pour une résolution de 12 bits.
int16_t ds18b20ReadTemperature(DS18B20Sensor *sensor) {
	// La communication commence par un reset.
	onewireReset(sensor->oneWireBus);
	
	// On sélectionne notre DS18B20.
	onewireMatchROM(sensor->oneWireBus, sensor->_romCode);
	
	// On lance la conversion.
	onewireSendCommand(sensor->oneWireBus, DS18B20_convertT);

	// Et on attend qu'elle soit terminée.
	onewireWaitForCompletion(sensor->oneWireBus);
	
	// On commence par un reset.
	onewireReset(sensor->oneWireBus);
	
	// On sélectionne la DS18B20.
	onewireMatchROM(sensor->oneWireBus, sensor->_romCode);
	
	// Et on lit le résultat de la conversion.
	onewireSendCommand(sensor->oneWireBus, DS18B20_readScratchpad);
	onewireReceiveBytes(sensor->oneWireBus, sensor->_buffer, sizeof(sensor->_buffer));
	
	// La température est dans les 2 premiers octets du tampon, octet de poids
	// faible en premier. C'est un nombre signé à virgule fixe placée entre les
	// bits 4 et 3. Cela signifie que pour avoir la partie entière de la
	// température, il faut décaler le résultat de 4 bits à droite.
	return (int16_t) (((uint16_t) sensor->_buffer[1]) << 8 | sensor->_buffer[0]);
}
