#ifndef _DS12B20_H
#define _DS12B20_H

/**
 * @file DS18B20.h
 * 
 * Driver pour le capteur de température DS18B20.
 * 
 * **IMPORTANT :** ce driver suppose qu'il n'y a qu'une seule DS18B20
 * sur le bus 1-Wire, qu'elle est alimentée au moyen de sa broche VDD,
 * et qu'une résistance pull-up de 4.7kΩ est connectée à sa broche DQ
 * (connectée au bus 1-Wire).
 */

#include "onewire.h"

/**
 * Précision de la conversion.
 * 
 * Plus la précision est grande, plus la conversion prend de temps.
 */
typedef enum {
	DS18B20_9bit = 0,  /**< Temps de conversion maximum = 93.75ms, Précision = 0.5°C */
	DS18B20_10bit = 1, /**< Temps de conversion maximum = 187.5ms, Précision = 0.25°C */
	DS18B20_11bit = 2, /**< Temps de conversion maximum = 375ms,   Précision = 0.125°C */
	DS18B20_12bit = 3, /**< Temps de conversion maximum = 750ms,   Précision = 0.0625°C */
} DS18B20Precision;

/**
 * Configuration et données d'état du capteur.
 * 
 * **IMPORTANT :** les symboles dont le nom commence par 2 underscores sont
 * réservés à l'usage interne du driver.
 */
typedef struct {
	OneWireConfig *oneWireBus; /**< Configuration du bus 1-Wire auquel la
		DS18B20 est reliée. DOIT être initialisé avant d'utiliser la DS18B20. */
	uint8_t _romCode[ONEWIRE_ROMCODE_SIZE]; /**< Identifiant unique de la DS18B20. */
	uint8_t _buffer[9];  /**< Tampon utilisé pour la communication. */
} DS18B20Sensor;

/**
 * Vérifie la présence de la DS18B20 et configure la précision de la conversion.
 * 
 * **IMPORTANT :** onewireInitialise() doit avoir été appelé AVANT d'utiliser
 * ds18b20Initialise().
 *
 * @param sensor Adresse du descripteur de la DS18B20.
 * @param precision Précision de conversion désirée.
 *
 * @return true si l'initialisation a réussi, false s'il n'y a aucun
 * périphérique sur le bus 1-Wire (ou si la résistance pull-up manque).
 */
bool ds18b20Initialise(DS18B20Sensor *sensor, DS18B20Precision precision);

/**
 * Démarre une conversion et retourne son résultat sous la forme d'un nombre
 * à virgule fixe avec le point décimal entre les bits 4 et 3. Si vous voulez
 * uniquement la partie entière de la température, vous devrez donc diviser
 * le résultat par 16.
 *
 * @param sensor Adresse du descripteur de la DS18B20.
 * 
 * @return La température en degrés centigrades.
 */
int16_t ds18b20ReadTemperature(DS18B20Sensor *sensor);

#endif // _DS12B20_H
