#include "project-defs.h"
#include "DS18B20.h"

// Déclaration du bus 1-Wire.
static OneWireConfig _ONE_WIRE_BUS = {
	.port = GPIOC,
	.pin = GPIO_Pin_4,
};

// Déclaration de la DS18B20 et de sa connexion au bus 1-Wire.
static DS18B20Sensor _DS18B20 = {
	.oneWireBus = &_ONE_WIRE_BUS,
};

/**
 * Convertit un nombre à virgule fixe en une chaîne de caractères.
 * Nécessaire lorsque l'implémentation de printf() ne gère pas les nombres à virgule flottante.
 *
 * @param value La valeur à convertir.
 * @param decimalBits Le nombre de bits après le point décimal.
 * @param signAlways Si true, le signe figurera toujours dans le résultat. Si false, le signe n'y figurera que s'il est négatif.
 * @param decimalPoint Le caractère à utiliser pour représenter le point décimal.
 * @param buffer Tampon pour le stockage de la chaîne.
 * @param bufferSize Taille totale du tampon.
 */
void fixedPointToString(int value, uint8_t decimalBits, bool signAlways, char decimalPoint, char *buffer, size_t bufferSize) {
	int i = 0;

	// On s'occupe du signe
	if (value < 0) {
		buffer[i++] = '-';
		value = -value;
	} else if (signAlways) {
		buffer[i++] = '+';
	}

	// On calcule la valeur qui précède le point décimal.
	int before = value >> decimalBits;

	// On calcule la valeur correspondant au premier bit après le point décimal
	// en tenant compte du nombre total de chiffres.
	int powerOfTwo = 5;

	for (int n = decimalBits - 1; n; n--) {
		powerOfTwo *= 10;
	}

	// On calcule la valeur qui suit le point décimal.
	int after = 0;

	for (int n = decimalBits; n; n--) {
		if (value & (1 << (n - 1))) {
			after += powerOfTwo;
		}

		powerOfTwo >>= 1;
	}

	// On convertit le tout en chaîne de caractères.
	snprintf(buffer + i, bufferSize - i, "%d%c%0*d", before, decimalPoint, decimalBits, after);
}

int main() {
	// Nécessaire pour le calcul des délais.
	SystemCoreClockUpdate();
	Delay_Init();

	// Nécessaire pour pouvoir utiliser printf().
	USART_Printf_Init(115200);

	// On initialise le bus 1-Wire.
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	onewireInitialise(&_ONE_WIRE_BUS);

	// On initialise le driver de la DS18B20.
	ds18b20Initialise(&_DS18B20, DS18B20_12bit);

	while (1) {
		// La DS18B20 chauffe en cours d'utilisation, ce qui affecte la
		// température lue. Il faut donc lui laisser le temps de refroidir
		// entre 2 mesures, d'où ce délai de 10 secondes.
		Delay_Ms(10000);

		// On lit la température
		int temp = ds18b20ReadTemperature(&_DS18B20);

		// Tampon pour la conversion du résultat.
		char buf[16];

		// Comme cette version de printf() ne supporte pas le format %f
		// pour afficher les nombres en virgule flottante, on doit gérer
		// nous-mêmes la conversion du résultat en virgule fixe en une
		// chaîne de caractères.
		fixedPointToString(temp, 4, true, '.', buf, sizeof(buf));

		// Et on affiche le résultat.
		printf("Température = %s°C\r\n", buf);
	}
}
