#include "project-defs.h"
#include "onewire.h"

/*
 * Codes des commandes génériques 1-wire.
 */
typedef enum {
	CMD_readROM = 0x33,
	CMD_matchROM = 0x55,
	CMD_searchROM = 0xf0,
	CMD_skipROM = 0xcc,
	CMD_alarmSearch = 0xec,
} OnewireCommand;

void onewireInitialise(OneWireConfig *config) {
	// On s'assure que le bus 1-Wire bus soit à l'état "idle".
	GPIO_WriteBit(config->port, config->pin, Bit_SET);

	// On configure la ligne de GPIO en sortie open-drain
	GPIO_InitTypeDef gpioInit = { 0 };
	gpioInit.GPIO_Pin = config->pin;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_OD;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(config->port, &gpioInit);
}

bool onewireReset(OneWireConfig *config) {
	// On met la ligne du bus à l'état bas.
	GPIO_WriteBit(config->port, config->pin, Bit_RESET);
	// La data sheet dit "au moins 480μs" pour provoquer le reset des esclaves.
	Delay_Us(480);
	// On remet la ligne du bus à l'état haut pour laisser la main aux esclaves.
	GPIO_WriteBit(config->port, config->pin, Bit_SET);
	// La data sheet dit "15 à 60μs" avant que l'esclave ne mette la ligne à
	// l'état bas pendant 60 à 240μs,donc lire l'état de la ligne après 70μs
	// devrait fonctionner dans tous les cas, c'est-à-dire à la fois pour
	// l'esclave le plus rapide (15μs + 60μs) et le plus lent (60μs + 240μs).
	Delay_Us(70);
	// Le bus devrait maintenant être à l'état bas si au moins un esclave est
	// présent et fonctionnel.
	bool result = GPIO_ReadInputDataBit(config->port, config->pin) == Bit_RESET;
	// La data sheet dit "au moins 480μs" à partir du moment où le maître met
	// la ligne du bus à l'état haut, mais nous avons déjà attendu 70μs, donc
	// reste 410.
	Delay_Us(410);
	
	return result;
}

/*
 * Ecriture d'un bit de donnée sur le bus.
 */
static void _writeBit(OneWireConfig *config, uint8_t bit) {
	// Ouvrir une fenêtre d'écriture (write slot).
	GPIO_WriteBit(config->port, config->pin, Bit_RESET);
	// La data sheet dit que le maître reste à l'état bas "au moins 1μs",
	Delay_Us(1);
	// On envoie maintenant le bit de donnée à l'esclave.
	GPIO_WriteBit(config->port, config->pin, bit ? Bit_SET : Bit_RESET);
	// La data sheet dit que la fenêtre d'écriture doit durer "au moins 60μs"
	Delay_Us(60);
	// Fermer la fenêtre d'écriture.
	GPIO_WriteBit(config->port, config->pin, Bit_SET);
}

/*
 * Lecture d'un bit de donnée sur le bus.
 */
static uint8_t _readBit(OneWireConfig *config) {
	// Ouvrir une fenêtre d'écriture (read slot).
	GPIO_WriteBit(config->port, config->pin, Bit_RESET);
	// La data sheet dit que le maître reste à l'état bas "au moins 1μs",
	Delay_Us(1);
	// On remet la ligne à l'état haut pour laisser la main à l'esclave.
	GPIO_WriteBit(config->port, config->pin, Bit_SET);
	// Le maître doit lire l'état de la ligne avant 15μs, donc 10 fait l'affaire.
	Delay_Us(10);
	// On lit l'état de la ligne.
	uint8_t result = (GPIO_ReadInputDataBit(config->port, config->pin) != Bit_RESET) ? 1 : 0;
	// La data sheet dit que la fenêtre de lecture doit durer "au moins 60μs",
	// mais nous avons déjà attendu 10μs, donc reste 50.
	Delay_Us(50);
	
	return result;
}

void onewireSendBytes(OneWireConfig *config, const uint8_t *buffer, int count) {
	for (int n = 0; n < count; n++) {
		uint8_t byte = buffer[n];
		
		for (int bit = 8; bit; bit --) {
			// Les octets sont lus bit le moins significatif (LSB) en premier.
			_writeBit(config, byte & 1);
			byte = byte >> 1;
		} 
	}
}

void onewireReceiveBytes(OneWireConfig *config, uint8_t *buffer, int count) {
	for (int n = 0; n < count; n++) {
		uint8_t byte = 0;
		
		for (int bit = 8; bit; bit --) {
			// Les octets sont lus bit le moins significatif (LSB) en premier,
			// d'où le 0x80 et le décalage à droite.
			byte = byte >> 1;
			byte |= _readBit(config) ? 0x80 : 0;
		}
		
		buffer[n] = byte;
	}
}

void onewireWaitForCompletion(OneWireConfig *config) {
	while (!_readBit(config));
}

void onewireSendCommand(OneWireConfig *config, uint8_t command) {
	onewireSendBytes(config, (const uint8_t *) &command, 1);
}

void onewireReadROM(OneWireConfig *config, uint8_t *romcode) {
	onewireSendCommand(config, CMD_readROM);
	onewireReceiveBytes(config, romcode, ONEWIRE_ROMCODE_SIZE);
}

void onewireMatchROM(OneWireConfig *config, const uint8_t *romcode) {
	onewireSendCommand(config, CMD_matchROM);
	onewireSendBytes(config, romcode, ONEWIRE_ROMCODE_SIZE);
}

void onewireSkipROM(OneWireConfig *config) {
	onewireSendCommand(config, CMD_skipROM);
}
