#ifndef _ONEWIRE_H
#define _ONEWIRE_H

/**
 * @file onewire.h
 * 
 * Driver pour le bus 1-Wire de Dallas Semiconductor (maintenant Analog Devices).
 */

/**
 * Configuration du bus 1-Wire.
 */
typedef struct {
	GPIO_TypeDef *port;
	uint16_t pin;
} OneWireConfig;

/**
 * Taille d'un identifiant unique d'esclave 1-wire, en octets.
 */
#define ONEWIRE_ROMCODE_SIZE 8

/**
 * Configure le bus 1-Wire.
 *
 * **IMPORTANT** : RCC_APB2PeriphClockCmd() doit être appelé AVANT d'utiliser
 * onewireInitialise().
 *
 * @param config Descripteur du bus 1-Wire.
 */
void onewireInitialise(OneWireConfig *config);

/**
 * Effectue un reset de tous les esclaves connectés au bus 1-Wire.
 *
 * @param config Descripteur du bus 1-Wire.
 * 
 * @returns true en cas de succès, false si aucun esclave n'est connecté au bus.
 */
bool onewireReset(OneWireConfig *config);

/**
 * Emet une séquence d'octets.
 *
 * @param config Descripteur du bus 1-Wire.
 * @param buffer Adresse de début des données à envoyer.
 * @param count Nombre d'octets à envoyer.
 */
void onewireSendBytes(OneWireConfig *config, const uint8_t *buffer, int count);

/**
 * Reçoit une séquence d'octets.
 *
 * @param config Descripteur du bus 1-Wire.
 * @param buffer Adresse du tampon de réception.
 * @param count Nombre d'octets attendus.
 */
void onewireReceiveBytes(OneWireConfig *config, uint8_t *buffer, int count);

/**
 * Attend la fin de l'exécution de la dernière commande. N'est nécessaire que
 * pour quelques commandes (ex. acquisition de température).
 * 
 * @param config Descripteur du bus 1-Wire.
 */
void onewireWaitForCompletion(OneWireConfig *config);

/**
 * Envoie une commande sur le bus 1-wire. Le but de cette fonction est uniquement de rendre le code plus lisible.
 *
 * @param config Descripteur du bus 1-Wire.
 * @param command Code de la commande.
 */
void onewireSendCommand(OneWireConfig *config, uint8_t command);

/**
 * Lit l'identifiant unique (aka. ROM code) du seul esclave présent sur le bus 1-Wire.
 *
 * @param config Descripteur du bus 1-Wire.
 * @param romcode Tampon recevant le ROM code lu. Doit avoir une taille de ONEWIRE_ROMCODE_SIZE octets.
 */
void onewireReadROM(OneWireConfig *config, uint8_t *romcode);

/**
 * Sélectionne l'esclave destinataire de la prochaine commande.
 *
 * @param config Descripteur du bus 1-Wire.
 * @param romcode ROM code de l'esclave concerné.
 */
void onewireMatchROM(OneWireConfig *config, const uint8_t *romcode);

/**
 * Signale aux esclaves présents sur le bus 1-Wire qu'ils devront tous traiter la prochaine commande.
 *
 * @param config Descripteur du bus 1-Wire.
 */
void onewireSkipROM(OneWireConfig *config);

#endif // _ONEWIRE_H
